// --------------------------------------------------------------------
// Copyright (c) 2005 by Terasic Technologies Inc. 
// --------------------------------------------------------------------
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// --------------------------------------------------------------------
//           
//                     Terasic Technologies Inc
//                     356 Fu-Shin E. Rd Sec. 1. JhuBei City,
//                     HsinChu County, Taiwan
//                     302
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// --------------------------------------------------------------------
//
// Major Functions:	DE2 LTM module Timing control and color pattern 
//					generator
//
// --------------------------------------------------------------------
//
// Revision History :
// --------------------------------------------------------------------
//   Ver  :| Author            		:| Mod. Date :| Changes Made:
//   V1.0 :| Johnny Fan				:| 07/06/30  :| Initial Revision
// --------------------------------------------------------------------
module lcd_timing_controller		(
						clk, 				// LCD display clock
						reset_n, 			// systen reset
						read,
						write,
						chipselect,
						address,
						readdata,
						writedata,
						//LCD SIDE
						oHD,				// LCD Horizontal sync 
						oVD,				// LCD Vertical sync 	
						oDEN,				// LCD Data Enable
						oLCD_R,				// LCD Red color data 
						oLCD_G,             // LCD Green color data  
						oLCD_B,             // LCD Blue color data  
						oVGA_ADDR,
						oVGA_WE_N,
						oVGA_OE_N,
						iVGA_DQ,
						iDISPLAY_MODE
						);
//============================================================================
// PARAMETER declarations
//============================================================================
parameter H_LINE = 1056;
parameter V_LINE = 525;
parameter Hsync_Blank = 216;
parameter Hsync_Front_Porch = 40;
parameter Vertical_Back_Porch = 35;
parameter Vertical_Front_Porch = 10;
//===========================================================================
// PORT declarations
//===========================================================================
input		clk;   
input		reset_n;
input		read;
input		write;
input		chipselect;
input [4:0]address;
output [15:0]readdata;
input [15:0]writedata;
output	[7:0]	oLCD_R;		
output  [7:0]	oLCD_G;
output  [7:0]	oLCD_B;
output			oHD;
output			oVD;
output			oDEN;
input	[1:0]	iDISPLAY_MODE;	
output [17:0] oVGA_ADDR;
output oVGA_WE_N;
output oVGA_OE_N;
input [15:0] iVGA_DQ;
//=============================================================================
// REG/WIRE declarations
//=============================================================================
reg		[10:0]  x_cnt;  
reg		[9:0]	y_cnt; 
wire	[7:0]	mred;
wire	[7:0]	mgreen;
wire	[7:0]	mblue; 
wire			display_area;
reg				mhd;
reg				mvd;
reg				mden;
reg				oHD;
reg				oVD;
reg				oDEN;
reg		[7:0]	oLCD_R;
reg		[7:0]	oLCD_G;	
reg		[7:0]	oLCD_B;	
wire	[1:0]	msel;	
reg		[7:0]	red_1;
reg 	[7:0]	green_1;
reg 	[7:0]	blue_1;
reg 	[7:0]	graycnt;
reg		[7:0]	pattern_data;
reg [31:0] div;
wire [15:0] area;
reg [23:0]oRGB_DATA; //information containing mole state
reg [2:0]mole0;
reg [2:0]mole1;
reg [2:0]mole2;
reg [2:0]mole3;
reg [2:0]mole4;
reg [2:0]mole5;
reg [2:0]mole6;
reg [2:0]mole7;
reg [2:0]mole8;
reg [2:0]mole9;
wire [7:0]LED1_R;
wire [7:0]LED1_G;
wire [7:0]LED1_B;
wire [7:0]LED2_R;
wire [7:0]LED2_G;
wire [7:0]LED2_B;
wire [7:0]LED0_R;
wire [7:0]LED0_G;
wire [7:0]LED0_B;
wire [2:0]oOVERRIDE;
reg [3:0] inumber_s;
reg [3:0] inumber_ts;
reg [3:0] inumber_min;
reg [26:0] counter;

wire [3:0] num_input_s;
wire [3:0] num_input_ts;
wire [3:0] num_input_min;
wire [26:0] oCNT;
wire flag1;
wire flag2;
wire clk_ltm;
assign clk_ltm = div[0]; // 25 Mhz
reg [14:0] area_counter[0:14];
//=============================================================================
// Structural coding
//=============================================================================
always @(posedge clk)
	begin
		div <= div+1;
	end
					
// This signal indicate the lcd display area .
assign	display_area = ((x_cnt>(Hsync_Blank-1)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch))
						))  ? 1'b1 : 1'b0;


///////////////////////// x  y counter  and lcd hd generator //////////////////
always@(posedge clk_ltm or negedge reset_n)
	begin
		if (!reset_n)
		begin
			x_cnt <= 11'd0;	
			mhd  <= 1'd0;  	
		end	
		else if (x_cnt == (H_LINE-1))
		begin
			x_cnt <= 11'd0;
			mhd  <= 1'd0;
		end	   
		else
		begin
			x_cnt <= x_cnt + 11'd1;
			mhd  <= 1'd1;
		end	
	end

always@(posedge clk_ltm or negedge reset_n)
	begin
		if (!reset_n)
			y_cnt <= 10'd0;
		else if (x_cnt == (H_LINE-1))
		begin
			if (y_cnt == (V_LINE-1))
				y_cnt <= 10'd0;
			else
				y_cnt <= y_cnt + 10'd1;	
		end
	end
////////////////////////////// touch panel timing //////////////////

always@(posedge clk_ltm  or negedge reset_n)
	begin
		if (!reset_n)
			mvd  <= 1'b1;
		else if (y_cnt == 10'd0)
			mvd  <= 1'b0;
		else
			mvd  <= 1'b1;
	end			

always@(posedge clk_ltm  or negedge reset_n)
	begin
		if (!reset_n)
			mden  <= 1'b0;
		else if (display_area)
			mden  <= 1'b1;
		else
			mden  <= 1'b0;
	end			


						
//Detect which area you are in// 0~14 different areas//offset corrected
assign area[0] = ((x_cnt>(Hsync_Blank-2+400)&& //>214
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-200))&& //< 1015
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-320))
						))  ? 1'b1 : 1'b0 ; //mole0
assign area[1] = ((x_cnt>(Hsync_Blank-2+200)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-400))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-320))
						))  ? 1'b1 : 1'b0;//mole1
assign area[2] = ((x_cnt>(Hsync_Blank-2)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-600))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-320))
						))  ? 1'b1 : 1'b0 ;//mole2
assign area[3] = ((x_cnt>(Hsync_Blank-2+400)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-200))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+160))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-160))
						))  ? 1'b1 : 1'b0 ;	//mole3
assign area[4] = ((x_cnt>(Hsync_Blank-2+200)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-400))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+160))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-160))
						))  ? 1'b1 : 1'b0 ; //mole4
assign area[5] = ((x_cnt>(Hsync_Blank-2)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-600))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+160))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-160))
						))  ? 1'b1 : 1'b0 ;//mole5
assign area[6] = ((x_cnt>(Hsync_Blank-2+400)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-200))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+320))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch))
						))  ? 1'b1 : 1'b0 ;	//mole6
assign area[7] = ((x_cnt>(Hsync_Blank-2+200)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-400))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+320))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch))
						))  ? 1'b1 : 1'b0; //mole7
assign area[8] = ((x_cnt>(Hsync_Blank-2)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1-600))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+320))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch))
						))  ? 1'b1 : 1'b0 ;//mole8
assign area[9] = ((x_cnt>(Hsync_Blank-2+600)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-400))
						))  ? 1'b1 : 1'b0 ;//area 9 Time
assign area[10] = ((x_cnt>(Hsync_Blank-2+600)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+80))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-320))
						))  ? 1'b1 : 1'b0 ;//area 10 life
assign area[11] = ((x_cnt>(Hsync_Blank-2+600)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+160))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-240))
						))  ? 1'b1 : 1'b0 ;//area 11 score
assign area[12] = ((x_cnt>(Hsync_Blank-2+600)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+240))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-160))
						))  ? 1'b1 : 1'b0 ;//area 12 start/pause
assign area[13] = ((x_cnt>(Hsync_Blank-2+600)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+320))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch-80))
						))  ? 1'b1 : 1'b0 ;//area 13 exit
assign area[14] = ((x_cnt>(Hsync_Blank-2+600)&& //>215
						(x_cnt<(H_LINE-Hsync_Front_Porch-1))&& //< 1016
						(y_cnt>(Vertical_Back_Porch-1+400))&& 
						(y_cnt<(V_LINE - Vertical_Front_Porch))
						))  ? 1'b1 : 1'b0 ;//area 14 label
assign area[15] = 1'b0;
//area 15 is the logo of the team defalt black

//////////////////////Timmer/////////////////////////
No_GEN LED0(				
				.iclk(clk_ltm),
				.iReset_n(reset_n),
				.ixcount(x_cnt),
				.iycount(y_cnt),
				.iled_pos_x(Hsync_Blank-1+600),
				.iled_pos_y(Vertical_Back_Porch),
				.inumber(num_input_s),
				.oOVERRIDE(oOVERRIDE[0]),
				.oR(LED0_R),
				.oG(LED0_G),
				.oB(LED0_B)
);

No_GEN LED1(				
				.iclk(clk_ltm),
				.iReset_n(reset_n),
				.ixcount(x_cnt),
				.iycount(y_cnt),
				.iled_pos_x(Hsync_Blank-1+640),
				.iled_pos_y(Vertical_Back_Porch),
				.inumber(num_input_ts),
				.oOVERRIDE(oOVERRIDE[1]),
				.oR(LED1_R),
				.oG(LED1_G),
				.oB(LED1_B)
);

No_GEN LED2(				
				.iclk(clk_ltm),
				.iReset_n(reset_n),
				.ixcount(x_cnt),
				.iycount(y_cnt),
				.iled_pos_x(Hsync_Blank-1+680),
				.iled_pos_y(Vertical_Back_Porch),
				.inumber(num_input_min),
				.oOVERRIDE(oOVERRIDE[2]),
				.oR(LED2_R),
				.oG(LED2_G),
				.oB(LED2_B)
);

assign flag1 = (oCNT == 26'd24999999) ? 1'b1 : 1'b0;
assign flag2 = (inumber_s == 0) ? 1'b1 : 1'b0;
assign num_input_s = inumber_s;
assign num_input_ts = inumber_ts;
assign num_input_min = inumber_min;
assign oCNT = counter;

always@(posedge clk_ltm or negedge reset_n)
begin
	if(!reset_n)
	begin
	inumber_min <= 4'd2;
	inumber_ts <= 4'd0;
	inumber_s <= 4'd0;
	counter <= 26'b0;
	end
	else if(flag1)
		begin
			if(flag2)
				begin
					if(inumber_ts != 0)
						begin
							inumber_ts <= inumber_ts - 1;
							inumber_s <= 4'd9;
							counter <= 26'b0;
						end
					else
						begin
							if(inumber_min != 0)
								begin
									inumber_min <= inumber_min -1;
									inumber_ts  <= 4'd5;
									inumber_s   <= 4'd9;
									counter <= 26'b0;
								end
							else
								begin
									inumber_min <= 4'd2;
									inumber_ts <= 4'd0;
									inumber_s <= 4'd0;
									counter <= 26'b0;
								end
						end
				end
			else
			begin
			inumber_s <= inumber_s-1;
			counter <= 26'b0;
			end
		end
	else
		counter <= counter +1;
end

assign oVGA_ADDR = area[0] ? area_counter[0] : 18'bZ;
assign oVGA_OE_N = area[0] ? 1'b1 : 1'b0;
assign oVGA_WE_N = 0;

///////////////Test --Allocate color to each block////////////////////////
always@(posedge clk_ltm or negedge reset_n)
	begin
	if (!reset_n)
		begin
		red_1 <= 8'd255;
		green_1 <= 8'd255;
		blue_1 <= 8'd255;   
		end
	else if (!(oOVERRIDE[2] || oOVERRIDE[1] || oOVERRIDE[0]))
		case(area)
			16'b0000000000000001: 
			begin 
			if(area_counter[0] == 15'd31999)
				area_counter[0] <= 0;
			else
				area_counter[0] <= area_counter[0] +1;
			red_1 <= (iVGA_DQ[15:11]<<3);
			green_1 <= (iVGA_DQ[10:5]<<2);
			blue_1 <= (iVGA_DQ[4:0]<<3);
			end//area 1
			16'b0000000000000010: begin red_1 <= 8'd255;green_1 <= 8'd128;blue_1 <= 8'd0;end
			16'b0000000000000100: begin red_1 <= 8'd255;green_1 <= 8'd255;blue_1 <= 8'd0;end
			16'b0000000000001000: begin red_1 <= 8'd128;green_1 <= 8'd255;blue_1 <= 8'd0; end
			16'b0000000000010000: begin red_1 <= 8'd0;green_1 <= 8'd255;blue_1 <= 8'd0;end
			16'b0000000000100000: begin red_1 <= 8'd0;green_1 <= 8'd255;blue_1 <= 8'd128;end
			16'b0000000001000000: begin red_1 <= 8'd0;green_1 <= 8'd255;blue_1 <= 8'd255;end
			16'b0000000010000000: begin red_1 <= 8'd0;green_1 <= 8'd128;blue_1 <= 8'd255;end
			16'b0000000100000000: begin red_1 <= 8'd0;green_1 <= 8'd0;blue_1 <= 8'd255;end
			16'b0000001000000000: begin red_1 <= 8'd127;green_1 <= 8'd0;blue_1 <= 8'd255;end
			16'b0000010000000000: begin red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd255;end
			16'b0000100000000000: begin red_1 <= 8'd255;green_1 <= 8'd0;blue_1 <= 8'd127;end
			16'b0001000000000000: begin red_1 <= 8'd128;green_1 <= 8'd128;blue_1 <= 8'd128;end
			16'b0010000000000000: begin red_1 <= 8'd0;green_1 <= 8'd0;blue_1 <= 8'd0;end
			16'b0100000000000000: begin red_1 <= 8'd255;green_1 <= 8'd255;blue_1 <= 8'd255;end
			16'b1000000000000000: begin red_1 <= 8'd255;green_1 <= 8'd204;blue_1 <= 8'd204;end
			default: begin red_1 <= 8'd255;green_1 <= 8'd255;blue_1 <= 8'd255;	end
		endcase
	else
		begin
			case(oOVERRIDE)
				3'b001:
					begin
						red_1 <= LED0_R;
						green_1 <= LED0_G;
						blue_1 <= LED0_B;
					end
				3'b010:
					begin
						red_1 <= LED1_R;
						green_1 <= LED1_G;
						blue_1 <= LED1_B;
					end
				3'b100:
					begin
						red_1 <= LED2_R;
						green_1 <= LED2_G;
						blue_1 <= LED2_B;
					end
			endcase	
		end
	end
	
assign mred = red_1;
assign mgreen = green_1;
assign mblue  = blue_1;	

						
always@(posedge clk_ltm or negedge reset_n)
	begin
		if (!reset_n)
			begin
				oHD	<= 1'd0;
				oVD	<= 1'd0;
				oDEN <= 1'd0;
				oLCD_R <= 8'd0;
				oLCD_G <= 8'd0;
				oLCD_B <= 8'd0;
			end
		else
			begin
				oHD	<= mhd;
				oVD	<= mvd;
				oDEN <= display_area;
				oLCD_R <= mred;
				oLCD_G <= mgreen;
				oLCD_B <= mblue;
			end		
	end
						
endmodule



//////////////////////Number Display Module///////////////////////////
module No_GEN(
				iclk,
				iReset_n,
				ixcount,
				iycount,
				iled_pos_x,
				iled_pos_y,
				inumber,
				oOVERRIDE,
				oR,
				oG,
				oB
				);

//Port IO
input iclk;
input iReset_n;
input [3:0]inumber;
input [10:0] ixcount;
input [9:0]  iycount;
input [10:0] iled_pos_x;
input [10:0] iled_pos_y;
output oOVERRIDE;
output [7:0] oR;
output [7:0] oG;
output [7:0] oB;

//Wire & Reg
reg [7:0]  oR;
reg [7:0]  oG;
reg [7:0]  oB;
reg oOVERRIDE;
wire display_area;
wire number_area;
wire a;
wire b;
wire c;
wire d;
wire e;
wire f;
wire g;
wire a_en;
wire b_en;
wire c_en;
wire d_en;
wire e_en;
wire f_en;
wire g_en;

//Structure
assign display_area = ((ixcount > (iled_pos_x -1)) 
							&& (ixcount < (iled_pos_x+ 60) ) 
							&& (iycount > (iled_pos_y -1))
							&& (iycount < (iled_pos_y +80))) ? 1'b1:1'b0;
							
assign number_area = ((ixcount > (iled_pos_x +4)) 
							&& (ixcount < (iled_pos_x+ 35) ) 
							&& (iycount > (iled_pos_y + 9))
							&& (iycount < (iled_pos_y +70))) ? 1'b1:1'b0;
							
assign a = ((ixcount > (iled_pos_x + 4)) 
				&& (ixcount < (iled_pos_x + 35)) 
				&&(iycount > (iled_pos_y + 9))
				&&(iycount < (iled_pos_y + 18))) ? 1'b1 : 1'b0;
assign b = 	((ixcount > (iled_pos_x + 4)) 
				&& (ixcount < (iled_pos_x + 13)) 
				&&(iycount > (iled_pos_y + 9))
				&&(iycount < (iled_pos_y + 40))) ? 1'b1 : 1'b0;	
assign c =  ((ixcount > (iled_pos_x + 4)) 
				&& (ixcount < (iled_pos_x + 13)) 
				&&(iycount > (iled_pos_y + 39))
				&&(iycount < (iled_pos_y + 70))) ? 1'b1 : 1'b0;	
assign d = 	 ((ixcount > (iled_pos_x + 4)) 
				&& (ixcount < (iled_pos_x + 35)) 
				&&(iycount > (iled_pos_y + 61))
				&&(iycount < (iled_pos_y + 70))) ? 1'b1 : 1'b0;
assign e = 	 ((ixcount > (iled_pos_x + 26)) 
				&& (ixcount < (iled_pos_x + 35)) 
				&&(iycount > (iled_pos_y + 39))
				&&(iycount < (iled_pos_y + 70))) ? 1'b1 : 1'b0;
assign f = 	 ((ixcount > (iled_pos_x + 26)) 
				&& (ixcount < (iled_pos_x + 35)) 
				&&(iycount > (iled_pos_y + 9))
				&&(iycount < (iled_pos_y + 40))) ? 1'b1 : 1'b0;	
assign g = 	 ((ixcount > (iled_pos_x + 4)) 
				&& (ixcount < (iled_pos_x + 35)) 
				&&(iycount > (iled_pos_y + 35))
				&&(iycount < (iled_pos_y + 44))) ? 1'b1 : 1'b0;	

//assign a_en = ((inumber != 1) && (inumber != 4)) ? 1'b1 : 1'b0;
//assign b_en = ((inumber != 5) && (inumber != 6)) ? 1'b1 : 1'b0;
//assign c_en = ((inumber != 2)) ? 1'b1 : 1'b0;
//assign d_en = ((inumber != 1) && (inumber != 4) && (inumber != 7)) ? 1'b1 : 1'b0;
//assign e_en = ((inumber == 2) || (inumber == 6) || (inumber == 8) || (inumber == 0)) ? 1'b1: 1'b0;
//assign f_en = ((inumber == 4) || (inumber == 5) || (inumber == 6) || (inumber == 8) 
//					|| (inumber == 9) || (inumber == 0)) ? 1'b1: 1'b0;
//assign g_en = ((inumber != 1) && (inumber != 7) && (inumber != 0)) ? 1'b1 : 1'b0;
				
always @(posedge iclk or negedge iReset_n)
begin 
	if(!iReset_n)
			oOVERRIDE <= 1'b0;
	else 
	case(inumber)
		0: oOVERRIDE <= (a||b||c||d||e||f);
		1: oOVERRIDE <= (b||c);
		2: oOVERRIDE <= (a||b||g||e||d);
		3: oOVERRIDE <= (a||b||c||d||g);
		4: oOVERRIDE <= (b||c||f||g);
		5: oOVERRIDE <= (a||c||d||f||g);
		6: oOVERRIDE <= (a||c||d||e||f||g);
		7: oOVERRIDE <= (a||b||c);
		8: oOVERRIDE <= (a||b||c||d||e||f||g);
		9: oOVERRIDE <= (a||b||c||d||f||g);
		default : oOVERRIDE <= 1'b0;
	endcase
	oR <= 8'd255;
	oG <= 8'd255;
	oB <= 8'd255;
end
endmodule
			
	
	










